import { CommonTableColumn } from '~/common'
import type { CellSlotProps } from '../type'

type RowClassGetter = (param: {
  columns: CommonTableColumn[]
  rowData: any
  rowIndex: number
}) => string

type CellClassGetter = (param: CellSlotProps) => string

const defaultCellClassV2 = 'el-table-v2__cell-text'

export default function useTableStyle(tableName?: string) {
  const rowClass = ({ row, rowIndex }) => {
    const classes: string[] = []
    switch (tableName) {
      case 'flightContainer':
        if (rowIndex === 1) {
          classes.push('row-warning')
        }
        break
      case 'flightWaybill':
        if (rowIndex === 0) {
          classes.push('row-alarm')
        }
        if (rowIndex === 1) {
          classes.push('row-warning')
        }
        break
      default:
        break
    }
    return classes.join(' ')
  }

  const cellClass = ({ row, column, rowIndex, columnIndex }) => {
    const classes: string[] = []
    switch (tableName) {
      case 'flightWaybill':
        if (['C0', 'C1'].includes(column.property)) {
          classes.push('cell-click')
        }
        break
      case 'flightContainer':
        if (['C0'].includes(column.property)) {
          classes.push('cell-click')
        }
        break
      case 'waybillGoods':
        if (['C2'].includes(column.property)) {
          classes.push('cell-click')
        }
        break
      default:
        break
    }
    return classes.join(' ')
  }

  const rowClassV2: RowClassGetter = ({ columns, rowData, rowIndex }) => {
    const classes: string[] = []
    switch (tableName) {
      case 'airport':
        if (rowIndex <= 1) {
          classes.push('bg-gray')
        }
        if (rowIndex === 1) {
          classes.push('underline-red')
        }
        break
      default:
        break
    }
    return classes.join(' ')
  }

  const cellClassV2: CellClassGetter = ({
    column,
    columns,
    columnIndex,
    rowData,
    rowIndex,
  }) => {
    const classes: string[] = [defaultCellClassV2]
    if (['4/243', '0/6'].includes(rowData[column.columnName])) {
      classes.push('cell-warning')
    }
    if (['5/357'].includes(rowData[column.columnName])) {
      classes.push('cell-error')
    }
    if (['flightNO'].includes(column.columnName)) {
      classes.push('cell-click')
    }
    return classes.join(' ')
  }

  return {
    rowClass,
    cellClass,
    rowClassV2,
    cellClassV2,
  }
}
